/***************************************************************************
                          mmconfig.cpp  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mmconfig.h"
#include "musicmanconfig.h"
#include "../normalizationparameters.h"

#include <qcheckbox.h>
#include <qptrlist.h>
#include <qtextcodec.h>
#include <kconfig.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kcombobox.h>
#include <kglobal.h>
//#include <klocale.h>
#include <kcharsets.h>
#include <kgenericfactory.h>
#include <kaboutdialog.h>


typedef KGenericFactory<MMConfig, QWidget> KDEDFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_musicman, KDEDFactory( "kcmmusicman" ) );



MMConfig::MMConfig(QWidget *parent, const char *name, const QStringList dummy): MusicmanConfigDialog(parent,name) {
	config=new KConfig("musicman");
	setButtons(buttons());
	setupUI();
	load();

	// Useless...... just to stop anoying compiler "unused parameter" warning
	dummy.count();
}


void MMConfig::defaults() {
	preferUnderline->setChecked(true);
	uppercaseFirst->setChecked(true);
	digitsForTrack->setValue(2);
	extensionType->setCurrentItem(NormalizationParameters::extPlain);
	slashReplace->setText("\\");
}


void MMConfig::setupUI(void) {
	extensionType->insertItem(i18n("All lowercase (.mp3, .ogg, .flac, .wav)"),
				NormalizationParameters::extPlain);
	extensionType->insertItem(i18n("First Uppercase (.Mp3, .Ogg, .Flac, .Wav)"),
				NormalizationParameters::extUpperFirst);
	extensionType->insertItem(i18n("All UPPERCASE (.MP3, .OGG, .FLAC, .WAV)"),
				NormalizationParameters::extUpperAll);


	encoding->insertItem(i18n("Local encoding: ") + QString(KGlobal::locale()->codecForEncoding()->name()),0);
	encoding->insertStringList(KGlobal::charsets()->descriptiveEncodingNames());
	/*codecList = new QPtrList<QTextCodec>;
	QTextCodec *codec;
	for (int i = 0; (codec = QTextCodec::codecForIndex(i)); i++) {
		//codecList->append( codec );
		encoding->insertItem(codec->name(),i);
	}*/
}


void MMConfig::slotConfigChanged() {
	emit changed(true);
}


void MMConfig::load() {
	{
		KConfigGroupSaver reader(config,"Filename Layout");
		preferUnderline->setChecked(config->readBoolEntry("preferUnderline",true));
		uppercaseFirst->setChecked(config->readBoolEntry("uppercaseFirst",true));
		digitsForTrack->setValue(config->readNumEntry("digitsForTrack",2));
		extensionType->setCurrentItem(config->readNumEntry("extensionType",NormalizationParameters::extPlain));
		slashReplace->setText(config->readEntry("slashReplace","\\"));
	}
	{
		KConfigGroupSaver reader(config,"ID3");
		QString enc(config->readEntry("encoding"));
		if (enc.isNull()) {
			// Null is the current encoding from your $LANG environment
			encoding->setCurrentItem(0);
		} else {
			encoding->setCurrentText(KGlobal::charsets()->languageForEncoding(enc) + " ( " + enc + " )");
		}
	}
}



void MMConfig::save() {
	{
		KConfigGroupSaver saver(config,"Filename Layout");
		config->writeEntry("preferUnderline",preferUnderline->isChecked());
		config->writeEntry("uppercaseFirst",uppercaseFirst->isChecked());
		config->writeEntry("digitsForTrack",digitsForTrack->value());
		config->writeEntry("extensionType",extensionType->currentItem());
		config->writeEntry("slashReplace",slashReplace->text());
	}
	{
		KConfigGroupSaver saver(config,"ID3");
		if (encoding->currentItem() == 0) {
			// 0 is current locale. If this is the selection, there is no need to write it.
			config->deleteEntry("encoding");
		} else {
			config->writeEntry("encoding",
				KGlobal::charsets()->encodingForName(encoding->currentText()));
		}
	}
	config->sync();
}



MMConfig::~MMConfig() {
	delete config;
}




QString MMConfig::quickHelp() const {
	return i18n("This is where you configure the Musicman plugin. Musicman appears in Konqueror's context menu when you have music files selected: MP3, Ogg, Wav.");
}



int MMConfig::buttons () {
	return KCModule::Default|KCModule::Apply;
}



void MMConfig::slotShowAbout() {
	KAboutDialog *dlg=new KAboutDialog(this,i18n("KDE Music Manager"));
	dlg->setAuthor("Avi Alkalay","avi@unix.sh","http://musicman.sf.net","Concept, design and development");
	dlg->setVersion("");
	dlg->show();
}


const KAboutData* MMConfig::aboutData() {
	KAboutData *about = new KAboutData(i18n("kcmmusicman"),
		i18n("KDE Music Manager"), 0, 0, KAboutData::License_LGPL,
		i18n("(c) 2003 Avi Alkalay"));

	about->addAuthor("Avi Alkalay", 0, "avi@unix.sh");
	return about;
}



// --------------------------------------------------------------------------
/*
extern "C" {
	KCModule *create_Musicman(QWidget *parent, const char *name) {
		KGlobal::locale()->insertCatalogue("Musicman");
		return new MMConfig(parent, name);
	}
}
*/
