/***************************************************************************
                          masstagger.cpp  -  description
                             -------------------
    begin                : Mon Sep 22 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>
#include <kcombobox.h>
#include <ktextedit.h>
#include <qgroupbox.h>
#include <kpushbutton.h>
#include <qcheckbox.h>
#include <kfiledialog.h>

#include "musicfilenamelayout.h"
#include "musicinfo.h"
#include "masstagger.h"
#include "masstag.h" // the dialog from QT designer

#define MAXGENRE 147
const char *typegenre [MAXGENRE+2] = {
   "Blues","Classic Rock","Country","Dance","Disco","Funk","Grunge",
   "Hip-Hop","Jazz","Metal","New Age","Oldies","Other","Pop","R&B",
   "Rap","Reggae","Rock","Techno","Industrial","Alternative","Ska",
   "Death Metal","Pranks","Soundtrack","Euro-Techno","Ambient",
   "Trip-Hop","Vocal","Jazz+Funk","Fusion","Trance","Classical",
   "Instrumental","Acid","House","Game","Sound Clip","Gospel","Noise",
   "Alt. Rock","Bass","Soul","Punk","Space","Meditative",
   "Instrumental Pop","Instrumental Rock","Ethnic","Gothic",
   "Darkwave","Techno-Industrial","Electronic","Pop-Folk","Eurodance",
   "Dream","Southern Rock","Comedy","Cult","Gangsta Rap","Top 40",
   "Christian Rap","Pop/Funk","Jungle","Native American","Cabaret",
   "New Wave","Psychedelic","Rave","Showtunes","Trailer","Lo-Fi",
   "Tribal","Acid Punk","Acid Jazz","Polka","Retro","Musical",
   "Rock & Roll","Hard Rock","Folk","Folk/Rock","National Folk",
   "Swing","Fast-Fusion","Bebob","Latin","Revival","Celtic",
   "Bluegrass","Avantgarde","Gothic Rock","Progressive Rock",
   "Psychedelic Rock","Symphonic Rock","Slow Rock","Big Band",
   "Chorus","Easy Listening","Acoustic","Humour","Speech","Chanson",
   "Opera","Chamber Music","Sonata","Symphony","Booty Bass","Primus",
   "Porn Groove","Satire","Slow Jam","Club","Tango","Samba",
   "Folklore","Ballad","Power Ballad","Rhythmic Soul","Freestyle",
   "Duet","Punk Rock","Drum Solo","A Cappella","Euro-House",
   "Dance Hall","Goa","Drum & Bass","Club-House","Hardcore","Terror",
   "Indie","BritPop","Negerpunk","Polsk Punk","Beat",
   "Christian Gangsta Rap","Heavy Metal","Black Metal","Crossover",
   "Contemporary Christian","Christian Rock","Merengue","Salsa",
   "Thrash Metal","Anime","JPop","Synthpop",""
};

MassTagger::MassTagger(KFileItemList &_fileList, FilenameMiner *_miner, QWidget *_parent) {
	parent=_parent;
	fileList=_fileList;
	miner=_miner;

	userDialog();
}

MassTagger::~MassTagger() {
	delete massTaggerDialog;
}



void MassTagger::userDialog() {
	QString fromFilename(i18n("[Guess from filename]"));
	QString empty;
	QString title;

	// Create it
	massTaggerDialog=new MassTaggerDialog(parent);

	// Connect to the method that lets the user copy ID3s from other file
	connect(massTaggerDialog->copyTags,SIGNAL(clicked()),SLOT(getTemplateFile()));

	// Connect dialog results to some actions
	connect(massTaggerDialog->ok,SIGNAL(clicked()),SLOT(accept()));
	connect(massTaggerDialog->cancel,SIGNAL(clicked()),SLOT(finish()));


	title=massTaggerDialog->groupID3->title().arg(fileList.count());
	massTaggerDialog->groupID3->setTitle(title);
	
//	massTaggerDialog->iTitle->insertItem(empty,0);
//	massTaggerDialog->iArtist->insertItem(empty,0);
//	massTaggerDialog->iTrack->insertItem(empty,0);
//	massTaggerDialog->iAlbum->insertItem(empty,0);

	// Set some default initial values in some fields
	massTaggerDialog->iTitle->insertItem(fromFilename/*,1*/);
	massTaggerDialog->iArtist->insertItem(fromFilename/*,1*/);
	massTaggerDialog->iTrack->insertItem(fromFilename/*,1*/);
	massTaggerDialog->iAlbum->insertItem(fromFilename/*,1*/);

  // Insert Genres (from kdemultimedia/kfile-plugins/mp3/mp3info.h)
	massTaggerDialog->iGenre->insertItem("");
	QStringList list;
	for (int i=0; i<=MAXGENRE; i++)
		list.append(typegenre[i]);
	list.sort();
	massTaggerDialog->iGenre->insertStringList(list);
  
	// Make the dialog appear
	massTaggerDialog->exec();
}




void MassTagger::accept() {
	KFileItem *file;
	MusicInfo dlg=musicInfoFromDialog(); // Music info from dialog input fields

	for (file=fileList.first();file;file=fileList.next()) {
		MusicInfo org; // Info from the original file
		MusicInfo fromFilename; // Info guessed from the filename
		KFileMetaInfo metaInfo=file->metaInfo();

		// Get info from original file
		org=MusicInfo(*file);

		// Guess some info from filename
		fromFilename=miner->mine(file->url().fileName());
		cleanFromFilename(fromFilename);

		// Merge objects
		org.merge(dlg);
		org.merge(fromFilename);

		org.updateMetaInfo(metaInfo);
		if (!metaInfo.isEmpty()) metaInfo.applyChanges();
	}
}





/** Returns the MusicInfo object based on the user input from the dialog. Unchecked items in the dialog will make empty fields in the MusicInfo object. */
MusicInfo MassTagger::musicInfoFromDialog(){
		MusicInfo dlg;
		QString fromFilename(i18n("[Guess from filename]"));
    
		// This first 4 gives the option to guess from filename
		if (massTaggerDialog->cTitle->isChecked() &&
		    massTaggerDialog->iTitle->currentText().stripWhiteSpace()!=fromFilename)
			dlg.title=massTaggerDialog->iTitle->currentText();

		if (massTaggerDialog->cArtist->isChecked() &&
		    massTaggerDialog->iArtist->currentText().stripWhiteSpace()!=fromFilename)
			dlg.artist=massTaggerDialog->iArtist->currentText();

		if (massTaggerDialog->cAlbum->isChecked() &&
		    massTaggerDialog->iAlbum->currentText().stripWhiteSpace()!=fromFilename)
			dlg.album=massTaggerDialog->iAlbum->currentText();

		if (massTaggerDialog->cTrack->isChecked() &&
		    massTaggerDialog->iTrack->currentText().stripWhiteSpace()!=fromFilename)
			dlg.track=massTaggerDialog->iTrack->currentText().toInt();

		if (massTaggerDialog->cYear->isChecked())
			dlg.year=massTaggerDialog->iYear->currentText().toInt();

		if (massTaggerDialog->cComment->isChecked())
			dlg.comment=massTaggerDialog->iComment->text();

		if (massTaggerDialog->cGenre->isChecked())
			dlg.genre=massTaggerDialog->iGenre->currentText();
      
		return dlg;
}


/** Write property of FilenameMiner * miner. */
void MassTagger::setMiner( FilenameMiner * _newVal) {
	miner = _newVal;
}

/** Remove the values that the user don't want guessed from the filename. */
void MassTagger::cleanFromFilename(MusicInfo &dlg) {
		// This first 4 gives the option to guess from filename
	QString fromFilename(i18n("[Guess from filename]"));
  
//	if (!massTaggerDialog->cTitle->isChecked() || massTaggerDialog->iTitle->currentItem()!=1)
	if (!massTaggerDialog->cTitle->isChecked() ||
	    massTaggerDialog->iTitle->currentText().stripWhiteSpace()!=fromFilename)
		dlg.title=QString::null;

//	if (!massTaggerDialog->cArtist->isChecked() || massTaggerDialog->iArtist->currentItem()!=1)
	if (!massTaggerDialog->cArtist->isChecked() ||
	    massTaggerDialog->iArtist->currentText().stripWhiteSpace()!=fromFilename)
		dlg.artist=QString::null;

//	if (!massTaggerDialog->cAlbum->isChecked() || massTaggerDialog->iAlbum->currentItem()!=1)
	if (!massTaggerDialog->cAlbum->isChecked() ||
	    massTaggerDialog->iAlbum->currentText().stripWhiteSpace()!=fromFilename)
		dlg.album=QString::null;

//	if (!massTaggerDialog->cTrack->isChecked() || massTaggerDialog->iTrack->currentItem()!=1)
	if (!massTaggerDialog->cTrack->isChecked() ||
	    massTaggerDialog->iTrack->currentText().stripWhiteSpace()!=fromFilename)
		dlg.track=-1;

	if (!massTaggerDialog->cYear->isChecked())
		dlg.year=-1;

	if (!massTaggerDialog->cComment->isChecked())
		dlg.comment=QString::null;

	if (!massTaggerDialog->cGenre->isChecked())
		dlg.genre=QString::null;
}


/** A wrapper to the destructor in the form of a slot. */
void MassTagger::finish() {
	delete this;
}


/** Called when user chooses to copy ID3 tags from other file. Fills the dialog with the values encontered. */
void MassTagger::getTemplateFile() {
	KURL choosed;
	QString baseDir=fileList.first()->url().directory();

	choosed=KFileDialog::getOpenURL(baseDir,"audio/x-mp3 application/x-ogg",
		massTaggerDialog,i18n("Select file to copy ID3 tags from"));

	if (!choosed.isEmpty()) {
		KFileItem fi(choosed,QString::null,0);
		MusicInfo info(fi);

		if (!info.artist.isEmpty())
//			massTaggerDialog->iArtist->insertItem(info.artist);
			massTaggerDialog->iArtist->setCurrentText(info.artist);
		else massTaggerDialog->iArtist->setCurrentText("");
		if (!info.title.isEmpty())
//			massTaggerDialog->iTitle->insertItem(info.title);
			massTaggerDialog->iTitle->setCurrentText(info.title);
		else massTaggerDialog->iTitle->setCurrentText("");
		if (!info.album.isEmpty())
//			massTaggerDialog->iAlbum->insertItem(info.album);
			massTaggerDialog->iAlbum->setCurrentText(info.album);
		else massTaggerDialog->iAlbum->setCurrentText("");
		if (!info.comment.isEmpty())
			massTaggerDialog->iComment->setText(info.comment);
		else massTaggerDialog->iComment->setText("");
		if (info.track!=-1)
//			massTaggerDialog->iTrack->insertItem(QString::number(info.track));
			massTaggerDialog->iTrack->setCurrentText(QString::number(info.track));
		else massTaggerDialog->iTrack->setCurrentText("");
		if (info.year!=-1)
//			massTaggerDialog->iYear->insertItem(QString::number(info.year));
			massTaggerDialog->iYear->setCurrentText(QString::number(info.year));
		else massTaggerDialog->iYear->setCurrentText("");
		if (!info.genre.isEmpty())
		{
			int i;    
			for (i=0; i<massTaggerDialog->iGenre->count(); i++)
			{
				if (massTaggerDialog->iGenre->text(i) == info.genre)
				{
					massTaggerDialog->iGenre->setCurrentItem(i);
					break;
				}
			}
			if (i >= massTaggerDialog->iGenre->count())
				massTaggerDialog->iGenre->setCurrentItem(0);
		}
		else massTaggerDialog->iGenre->setCurrentItem(0);          
	}
}
