/***************************************************************************
                          musicinfo.cpp  -  description
                             -------------------
    begin                : Sat Jun 21 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "musicinfo.h"
#include "musicfilenamelayout.h"
#include <kfileitem.h>
#include <kfilemetainfo.h>
#include <kdebug.h>


/** Constructor from a KFileMetaInfo object */
MusicInfo::MusicInfo(KFileMetaInfo &metaInfo) {
	fromMetaInfo(metaInfo);
}

/** Constructor from a KFileItem object */
MusicInfo::MusicInfo(KFileItem &file) {
	KFileMetaInfo minfo=file.metaInfo();
	extension=MusicFilenameLayout::getExtension(file.url().fileName());
	fromMetaInfo(minfo);
}


void MusicInfo::fromMetaInfo(KFileMetaInfo &metaInfo) {
//	  kdDebug(0) << "fromMetaInfo: " << endl;
	if (!metaInfo.isEmpty()) {
		if (metaInfo.item("Tracknumber").isValid())
			track=metaInfo.item("Tracknumber").value().toInt();
		else track=-1;      
		if (metaInfo.item("Date").isValid())
//			year=metaInfo.item("Year").value().toInt();
			year=metaInfo.item("Date").value().toInt();
		else year=-1;      
		if (metaInfo.item("Artist").isValid())      
			artist=metaInfo.item("Artist").value().toString();
		else artist=QString::null;      
		if (metaInfo.item("Title").isValid())      
			title=metaInfo.item("Title").value().toString();
		else title=QString::null;      
		if (metaInfo.item("Album").isValid())      
			album=metaInfo.item("Album").value().toString();
		else album=QString::null;
		if (metaInfo.item("Comment").isValid())      
			comment=metaInfo.item("Comment").value().toString();
		else comment=QString::null;
		if (metaInfo.item("Bitrate").isValid())    
			bitrate=metaInfo.item("Bitrate").value().toInt();
		else bitrate=-1;
		if (metaInfo.item("Genre").isValid())      
			genre=metaInfo.item("Genre").value().toString();
		else genre=QString::null;

//	  kdDebug(0) << "fromMetaInfo: Title: " << title << endl;
	}
}

MusicInfo::MusicInfo() {
	year=-1;
	track=-1;
}

MusicInfo::~MusicInfo() {
}

/** Returns true if empty. */
bool MusicInfo::isNull() {
	if (album.isNull() && artist.isNull() && title.isNull() &&
		lyrics.isNull() && comment.isNull() && genre.isNull() &&
		track<0 && year<0) return true;
	
	return false;
}

/** Updates the passed metaInfo with the MusicInfo internal info.
@param metaInfo The metaInfo that will be updated. */
void MusicInfo::updateMetaInfo(KFileMetaInfo &metaInfo) {
		if (isNull()) return;

		if (!metaInfo.isEmpty()) {
			if (track>-1)
				metaInfo.item("Tracknumber").setValue(track);
 	
			if (year>-1)
//				metaInfo.item("Year").setValue(year);
				metaInfo.item("Date").setValue(year);
 	
			if (!artist.isNull())
				metaInfo.item("Artist").setValue(artist);
 	
			if (!title.isNull())
				metaInfo.item("Title").setValue(title);
 	
			if (!album.isNull())
				metaInfo.item("Album").setValue(album);
 	
			if (!comment.isNull())
				metaInfo.item("Comment").setValue(comment);

      if (!genre.isNull())
        metaInfo.item("Genre").setValue(genre);
		}
}





/** Updates info with the non-null info from newInfo. The empty (empty!=null) strings are updated. */
void MusicInfo::merge(MusicInfo &newInfo) {
		if (newInfo.isNull()) return;

		if (newInfo.track>-1)
			track=newInfo.track;

		if (newInfo.year>-1)
			year=newInfo.year;

		if (!newInfo.artist.isNull())
			artist=newInfo.artist;

		if (!newInfo.title.isNull())
			title=newInfo.title;

		if (!newInfo.album.isNull())
			album=newInfo.album;

		if (!newInfo.comment.isNull())
			comment=newInfo.comment;

		if (!newInfo.extension.isNull())
			extension=newInfo.extension;

		if (!newInfo.lyrics.isNull())
			lyrics=newInfo.lyrics;

		if (newInfo.bitrate>0)
			bitrate=newInfo.bitrate;

    if (!newInfo.genre.isNull())
      genre=newInfo.genre;
}
