/***************************************************************************
                          musicinfo.h  -  description
                             -------------------
    begin                : Sat Jun 21 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MUSICINFO_H
#define MUSICINFO_H

#include <qstring.h>

class KFileMetaInfo;
class KFileItem;

/**Encapsulates general music information: Artist, Title, Album, Tracknumber, Year, Composers, Performers, Comment, Genre, Lyrics
  *@author Avi Alkalay
  */

class MusicInfo {
	public: 
		/** Constructor from a KFileMetaInfo object */
		MusicInfo(KFileMetaInfo &metaInfo);
		/** Constructor from a KFileItem object */
		MusicInfo(KFileItem &file);
		MusicInfo();
		~MusicInfo();

		/** Returns true if empty. */
		bool isNull();

		/** Updates the passed metaInfo with the MusicInfo internal info. It will not write the mate info back to the file.
		@param metaInfo The metaInfo that will be updated. */
		void updateMetaInfo(KFileMetaInfo &metaInfo);

		/** Updates this object from values inside a KFileMetaInfo */
		void fromMetaInfo(KFileMetaInfo &metainfo);
  /** Updates info with the non-null info from newInfo. The empty (empty!=null) strings are updated. */
  void merge(MusicInfo &newInfo);

	public: // Public attributes
		/** Song title. */
		QString title;

		/** Song general artist. */
		QString artist;

		/** Song album. */
		QString album;

		/** Filename extension (mp3, ogg, etc). */
		QString extension;

		/** Song track. */
		int track;

		/** Bitrate quality. */
		int bitrate;

		/** Song year. */
		int year;

		/** Song comment. */
		QString comment;

		/** Song lyrics. */
		QString lyrics;

    /** Song genre */
    QString genre;
};

#endif

