/***************************************************************************
                          musicman.h  -  description
                             -------------------
    begin                : Tue May 27 21:25:35 BRT 2003
    copyright            : (C) 2003 by Avi Alkalay
    email                : avi@unix.sh
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef __plugin_musicman_h
#define __plugin_musicman_h
 
#include <konq_popupmenu.h>
#include <kdirlister.h>

class KAction;
class KConfig;

class MusicIndexGenerator;
class MassTagger;
class CustomRenamer;

#include "musicfilenamelayout.h"

/**
 * A plugin is the way to add actions to an existing @ref KParts application,
 * or to a @ref Part.
 *
 * The XML of those plugins looks exactly like of the shell or parts,
 * with one small difference: The document tag should have an additional
 * attribute, named "library", and contain the name of the library implementing
 * the plugin.
 *
 * If you want this plugin to be used by a part, you need to
 * install the rc file under the directory
 * "data" (KDEDIR/share/apps usually)+"/instancename/kpartplugins/"
 * where instancename is the name of the part's instance.
 **/
class MusicMan : public KonqPopupMenuPlugin {
	Q_OBJECT
public:
	MusicMan(KonqPopupMenu *, const char *name, const QStringList &list);
	virtual ~MusicMan();
	KonqPopupMenu *popup;
	KDirLister lister;

	/** This is the guy responsible for searching if a song filename matches some layout. */
	FilenameMiner *miner;
 
public slots:
	void slotToUnderline();
	void slotToSpace();
  /** Rename filename using ID3 values to match the layout specified by idx index in the miner (FilenameMiner). */
  void slotFilenameFromID3(int idx);
  /** Sets ID3 with info guessed from filename. */
	void slotID3FromFilename();
	/** Rename selected files from Latin1 to UTF-8 */
	void slotRenameLatin1ToUTF8();
	/** Creates the CD index, including HTML, Text, and the cover. */
	void slotIndex();
	/** Rename the file to a standard layout. */
	void slotNormalize();
	/** Remove the placeholder and put our submenus. */
	void slotAboutToShow();
  /** Calls the dialog for advanced ID3 tagging */
  void slotMassTag();
  /** Calls the Custom File Renaming dialog, for filename renaming based on templates */
  void slotCustomRename();


protected:
	void setupActions();

private:
	KConfig *config;
	KAction *actionDummy;
	/** Placeholder for Music Index Generator */
	MusicIndexGenerator * indexer;
	/** Placeholder for the Mass Tagger engine */
	MassTagger *massTagger;
	/** Placeholder for the Custome Renaming engine */
	CustomRenamer *customRenamer;
	/** Submenu containing the Music Manager main actions. */
	QPopupMenu *menuMusicMan;
	/** Submenu containing filename layouts for renaming. */
	QPopupMenu *menuFilenameFromID3;
	/** Holds all user preferences regarding details of filenames */
	NormalizationParameters normalization;
	/** The encoder to use when writing ID3 tags */
	QTextCodec * id3Encoding;
	/** Actually renames selected files, substituting character 'from' by 'to'. */
	void renameReplace(char from,char to);
  /** @returns false if there is files selected
@returns true if there is only folders selected */
  bool onlyFolders();

private slots: // Private slots
	/** Void slot */
	void slotPlaceholder();
};
 
#endif
