// g++ `taglib-config --libs --cflags` -I${QTDIR}/include -L${QTDIR}/lib -Wl,-R${QTDIR}/lib -lqt-mt tag-unbreaker.cpp -o tag-unbreaker

/* Copyright (C) 2003 Scott Wheeler <wheeler@kde.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <iostream>

#include <tstring.h>
#include <mpegfile.h>
#include <id3v2tag.h>
#include <id3v1tag.h>
#include <id3v2framefactory.h>

#include <qstring.h>
#include <qtextcodec.h>

TagLib::String convert(const TagLib::String &s)
{
  const char *encoding = getenv("TAG_ENCODING");

  QTextCodec *codec = encoding ?
    QTextCodec::codecForName(encoding) :
    QTextCodec::codecForLocale();

  if(!codec) {
    std::cerr << "Could not determine the appropriate locale." << std::endl;
    exit(1);
  }

  return QStringToTString(codec->toUnicode(s.toCString()).stripWhiteSpace());
}

int main(int argc, char *argv[])
{
  if(argc == 1) {
    std::cout << std::endl
	      << "tag-unbreaker - This small tool tried to unbreak some of the problems caused by" << std::endl
	      << "broken ID3v1 taggers, notable KDE's KFileMetaInfo plugin prior to KDE 3.2."      << std::endl
	      << std::endl
	      << "tag-unbreaker first converts the ID3v1 tag from the locale (the machine's"       << std::endl
	      << "locale if the environment variable TAG_ENCODING isn't set, or TAG_ENCODING"      << std::endl
	      << "if it is), strips the whitespace from the beginning and end and then writes a"   << std::endl
	      << "utf8 encoded ID3v2 tag and strips the broken ID3v1 tag.  (ID3v1 tags are"        << std::endl
	      << "to only contain ISO8859-1 data, but many broken taggers use the current locale." << std::endl
	      << std::endl
	      << "* WARNING *"                                                                     << std::endl
	      << std::endl
	      << "THIS WILL REALLY SCREW UP YOUR FILES IF YOU USE IT ON EITHER PROPPER ID3V1"      << std::endl
	      << "TAGS (ISO8859-1) OR FILES USING A DIFFERENT LOCALE THAN YOUR MACHINE'S."         << std::endl
	      << "YOU'VE BEEN WARNED."                                                             << std::endl
	      << std::endl;
  }


  TagLib::ID3v2::FrameFactory::instance()->setDefaultTextEncoding(TagLib::String::UTF8);

  for(int i = 1; i < argc; i++) {

    TagLib::MPEG::File f(argv[i], false);

    if(f.isValid()) {

      TagLib::Tag *target = f.ID3v2Tag(true);
      TagLib::Tag *source = f.ID3v1Tag(false);

      if(source) {

	std::cerr << "Fixing \"" << argv[i] << "\"" << std::endl;

	target->setTitle(convert(source->title()));
	target->setArtist(convert(source->artist()));
	target->setAlbum(convert(source->album()));
	target->setComment(convert(source->comment()));
	target->setGenre(convert(source->genre()));

	// No conversion needed.

	target->setYear(source->year());
	target->setTrack(source->track());

	f.save(TagLib::MPEG::File::ID3v2);
      }
      else
	std::cerr << "Could not fine ID3v1 tag in \"" << argv[i] << "\"" << std::endl;
    }
    else
      std::cerr << "Could not open \"" << argv[i] << "\"" << std::endl;
  }
}
